*:*********************************************************************
*:
*:        Program: TIMER.PRG
*:
*:         System: SYSTEM TIMER PROGRAM
*:         Author: JOHN G. BODE
*:      Copyright (c) 1991, JOHN G. BODE
*:  Last modified: 01/19/91     11:54
*:
*:  Procs & Fncts: HELP
*:               : NET_USE()
*:               : NOTIFY
*:               : REC_LOCK()
*:               : ADD_REC()
*:               : MEMOFUNC2()
*:               : FIL_LOCK()
*:
*:          Calls: HELP           (procedure in TIMER.PRG)
*:               : NET_USE()      (function  in TIMER.PRG)
*:               : NOTIFY         (procedure in TIMER.PRG)
*:               : REC_LOCK()     (function  in TIMER.PRG)
*:               : ADD_REC()      (function  in TIMER.PRG)
*:
*:           Uses: &BATPATH.DBF   
*:               : &IBATPATH.DBF  
*:               : &IEMPATH.DBF   
*:
*:      Documented 01/28/91 at 14:13                SNAP!  version 4.02d
*:*********************************************************************
*
* Timer.prg may be freely modified and/or distributed.
*
* Timer.exe will execute programs on a schedule based on time of day, 
* day of week, day of year, etc.  This program creates a batch file from 
* the contents of a memo field and then executes it.  I use timer.exe 
* on the Banyan network to send e-mail reminders at predetermined intervals.  
* It can be used on most any netwwork supported  by the Clipper compiler.  
* The timer program can be located in a batch file so that after execution 
* of an event it will return to the program to await another event.  This
* program is written for multiple user access.  On-line help is provided 
* which further explains the functions of this program.  
*
*
* SAMPLE BATCH SEQUENCE
*
* AUTOEXEC.BAT    STARTS SYSTEM FROM BOOTUP
*    
*  CHAINS TO:
*
* SUPER.BAT       EXECUTES TIMER.EXE FOLLOWED BY TIMECONT.BAT
*         (TIMER.EXE   TIMER PROGRAM)
*         (TIMECONT    TIMECONT IS CREATED BY TIMER.EXE FROM MEMO FIELDS)
*
*  CHAINS TO:
*
* SUPER.BAT       TIMECONT MUST CHANGE BACK TO SUPER.BAT TO LOAD TIMEPROG
*
*
* EACH RECORD ENTRY WILL ONLY EXECUTE ONE EVENT PER DAY
* EACH TIME FUNCTION WILL BE GIVEN A WINDOW OF n MINUTES TO EXECUTE.
*
*TIMEDATA.DBF = MAIN TABLE OF SCHEDULES
*TIMEDATA.DBT = CONTAINS MEMOFIELDS FOR THE ABOVE TABLE
*
*FIELDS: ALRMTIME    C 5     ALARM TIME
*        ALRMDATE    D       ALARM DATE
*        LASTTIME    C 5     TIME OF LAST ACTIVITY
*        LASTDATE    D       DATE OF LAST ACTIVITY
*        MONNUMY     N 2.0   MONTH OF YEAR HOLDING VARIABLE
*        DAYNUMM     N 2.0   DAY OF MONTH HOLDING VARIABLE
*        DAYNUMW     N 1.0   DAY OF WEEK HOLDING VARIABLE
*        NOTIFYC     N 2.0   METHOD OF NOTIFICATION CODE
*        PROGTYP     N 1.0   TYPE OF PROGRAM EXECUTION (E-MAIL OR BATCH)
*        BATCHT      MEMO    BATCH FILE TEXT
*        EMAILT      MEMO    E-MAIL TEXT
*        HELP        MEMO    HELP COMMENT WINDOW
*
***************************************************************************


************************** PARAMETER VARIABLES ****************************

WINDOW=120	&& TIME WINDOW IN MINUTES TO EXECUTE TIMED FUNCTION

loopdely=60	&& DELAY IN SECONDS BEFORE RECHECKING EVENTS LIST

strtdely=60     && DELAY IN SECONDS BEFORE TIMER AUTO STARTS

SET DEFAULT TO  C:\	 && TEXT FILE WILL BE WRITTEN TO DEFAULT PATH

filepath="C:\TIMEDATA" && DATA & MEMO FILE PATH

batpath="C:\TIMECONT.BAT"  && BATCH TEXT FILE OUTPUT PATH

empath="C:\TMP.TXT"  && EMAIL TEXT FILE OUTPUT PATH

ibatpath="C:\TIMECONT.TXT"  && BATCH TEXT FILE INPUT PATH

iempath="C:\TMPTXT.TXT"  && EMAIL TEXT FILE INPUT PATH

***************************************************************************

SET CONFIRM ON
SET DELETED ON
SET EXACT ON
SET COLOR TO GR+/BG,G+/N,BG,,G+/N
SET EXCLUSIVE OFF
CLEAR

* FIELD HOLDING VARIABLES
xemailn      = SPACE(15)  &&     E-MAIL DIST NAME
xalrmtime    = SPACE(5)   &&      ALARM TIME
xalrmdate    = CTOD("  /  /  ")  &&      ALARM DATE
xlasttime    = SPACE(5)    &&      TIME OF LAST ACTIVITY
xlastdate    = CTOD("  /  /  ")  &&      DATE OF LAST ACTIVITY
xmonnumy     = 0  &&   MONTH OF YEAR HOLDING VARIABLE
xdaynumm     = 0  &&   DAY OF MONTH HOLDING VARIABLE
xdaynumw     = 0  &&   DAY OF WEEK HOLDING VARIABLE
xnotifyc     = 0  &&    METHOD OF NOTIFICATION CODE
xprogtyp     = 0  &&    TYPE OF PROGRAM EXECUTION (E-MAIL OR BATCH)

sel=" "

SELECT 1
* USE TABLE(S) IN A MULTIUSER ENVIRONMENT
*  (OPEN FILE NON-EXCLUSIVE, RECORD LOCKING ENABLED)

IF net_use(filepath,.F., 5)
 *
ELSE
 SET COLOR TO GR+*/R
 @ 0,0 SAY "FILE ALREADY IN USE!     HIT ANY KEY TO CONTINUE!"
 SET COLOR TO GR+/BG,G+/N,BG
 INKEY(0)
 @ 0,0
ENDIF

GO TOP

* DISPLAY MENU

DO WHILE .T.       && MENU LOOP
 
 keypress=0
 
 @ 2,25 SAY "BATCH PROGRAM TIMER"
 
 @  8,8 SAY "B = BROWSE PROGRAM DATABASE"
 @ 10,8 SAY "T = START TIMER"
 @ 12,8 SAY "Q = QUIT TIMER PROGRAM (OR ESC)"
 
 @ 22,18 SAY "ALWAYS LEAVE COMPUTER AT THIS MENU"
 
 @ 18,28 SAY "ENTER SELECTION"
 
 @ 10,24 SAY "(AUTOMATICALLY STARTS IN     SECONDS)"
 
 cnt=strtdely
 DO WHILE keypress=0 .AND. cnt > 0
  keypress = INKEY(1)	&& ONE SECOND
  cnt=cnt-1
  @ 10,49 SAY STR(cnt,3)
 ENDDO
 @ 10,24 SAY SPACE(41)
 IF keypress=0
  keypress=ASC("T")
 ENDIF
 
 DO CASE        && TEST FOR MENU SELECTION
  
 CASE keypress=ASC("Q") .OR. keypress=ASC("q") .OR. keypress=27
  
  SET DEVICE TO PRINT		&& CREATE DEFAULT TIMECONT.BAT
  SET PRINT ON
  SET CONSOLE OFF
  IF FILE(batpath)=.T.
   ERASE &batpath
  ENDIF
  
  SET PRINTER TO &batpath
  ?? "ECHO OFF"
  ?  "CLS"
  ?  "ECHO  ONE MOMENT PLEASE!"
  ?  "MENUMAIN"
  ?  "MENUEXEC"
  ?
  
  SET PRINTER TO
  SET DEVICE TO SCREEN
  SET PRINT OFF
  SET CONSOLE ON
  
  CLOSE
  CLEAR
  QUIT
  
  * RUN THE TIMER
 CASE keypress=ASC("T") .OR. keypress=ASC("t")
  
  @ 10,24 SAY "(TIMER IS NOW RUNNING)"
  
  DO WHILE .T.      && TIMER LOOP - LOOP THRU DATABASE EVERY N SECONDS
   
   GO TOP
   DO WHILE EOF()=.F.  && LOOP THRU PROGRAM DATABASE
    
    IF LASTKEY()=27 .OR. LASTKEY()=ASC("Q") .OR. LASTKEY()=ASC("q")
     EXIT                   && LOOP THRU PROGRAM DATABASE
    ENDIF
    
    xalrmtime    =  a->alrmtime
    xalrmdate    =  a->alrmdate
    xlasttime    =  a->lasttime
    xlastdate    =  a->lastdate
    xmonnumy     =  a->monnumy
    xdaynumm     =  a->daynumm
    xdaynumw     =  a->daynumw
    xnotifyc     =  a->notifyc
    xprogtyp     =  a->progtyp
    
    * CONVERT TIMES INTO MINUTES
    currtime=(VAL(SUBSTR(TIME(),1,2))*60)+(VAL(SUBSTR(TIME(),4,2)))
    alrmtimem=(VAL(SUBSTR(xalrmtime,1,2))*60)+(VAL(SUBSTR(xalrmtime,4,2)))
    lasttimem=(VAL(SUBSTR(xlasttime,1,2))*60)+(VAL(SUBSTR(xlasttime,4,2)))
    
    DO CASE   && TEST NOTIFICATION SCHEDULE
     
    CASE xnotifyc = 0  && DO NOTHING WITH THIS RECORD
     
    CASE xnotifyc = 1   && TIME OF DAY, EVERY DAY
     IF (currtime >= alrmtimem .AND. currtime <= alrmtimem + WINDOW .AND. ;
       DATE() <> xlastdate)         && CHECK TIME OF DAY, EVERY DAY*****
      DO notify
     ENDIF
     
    CASE xnotifyc = 2 	&& DAY OF WEEK, EVERY WEEK
     IF (currtime >= alrmtimem .AND. currtime <= alrmtimem + WINDOW .AND. ;
       DATE() <> xlastdate .AND. xdaynumw = DOW(DATE())  )
      DO notify
     ENDIF
     
    CASE xnotifyc = 3   && DAY OF MONTH, EVERY MONTH
     IF (currtime >= alrmtimem .AND. currtime <= alrmtimem + WINDOW .AND. ;
       DATE() <> xlastdate .AND. DAY(DATE()) = xdaynumm)
      DO notify
     ENDIF
     
    CASE xnotifyc = 4   && DAY OF MONTH, ONE MONTH, EVERY YEAR
     IF (currtime >= alrmtimem .AND. currtime <= alrmtimem + WINDOW .AND. ;
       DATE() <> xlastdate .AND. MONTH(DATE()) = xmonnumy .AND. ;
       DAY(DATE()) = xdaynumm )
      DO notify
     ENDIF
     
    CASE xnotifyc = 5   && DATE, ONCE ONLY
     IF (currtime >= alrmtimem .AND. currtime <= alrmtimem + WINDOW .AND. ;
       DATE() <> xlastdate .AND. DATE() = xalrmdate)
      DO notify
     ENDIF
     
    CASE xnotifyc = 99   && IMMEDIATE ACTION, ONCE ONLY
     IF rec_lock(5)
      REPLACE a->notifyc     WITH 0
      COMMIT
      UNLOCK
     ELSE
      SET COLOR TO GR+*/R
      @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
      SET COLOR TO GR+/BG,G+/N,BG
      INKEY(0)
      @ 0,0
     ENDIF
     DO notify
     
    ENDCASE      && END TEST FOR TYPE OF NOTIFICATION
    
    SKIP 1
   ENDDO   && END LOOP THRU PROGRAM DATA BASE
   
   IF LASTKEY()=27
    EXIT    && TIMER LOOP
   ENDIF
   
   INKEY(loopdely)       && WAIT N SECONDS
  ENDDO	&& TIMER LOOP
  
  @ 10,24 SAY "                      "
  
 CASE keypress=ASC("B") .OR. keypress=ASC("b") && BROWSE THE TIMER DATABASE
  
  * DISPLAY FIELDS ON TOP, WINDOW1=BATCH PROGRAM, WINDOW2=INFO TO SEND E-MAIL,
  * DISPLAY RECORD NUMBER
  
  * THIS PROGRAM IS USED TO BROWSE AND/OR MODIFY THE TIMEDATA FILE.
  
  SET COLOR TO GR+/BG,G+/N,BG
  CLEAR
  SET DELETED OFF
  SET EXACT ON
  SET EXCLUSIVE OFF
  xdelete=.F.		&& LOCAL FLAG
  
  **************** INITIALIZE VARIABLES
  
  * FIELD HOLDING VARIABLES
  
  xalrmtime    = SPACE(5)   &&      ALARM TIME
  xalrmdate    = CTOD("  /  /  ")  &&      ALARM DATE
  xlasttime    = SPACE(5)    &&      TIME OF LAST ACTIVITY
  xlastdate    = CTOD("  /  /  ")  &&      DATE OF LAST ACTIVITY
  xmonnumy     = 0  &&   MONTH OF YEAR HOLDING VARIABLE
  xdaynumm     = 0  &&   DAY OF MONTH HOLDING VARIABLE
  xdaynumw     = 0  &&   DAY OF WEEK HOLDING VARIABLE
  xnotifyc     = 0  &&    METHOD OF NOTIFICATION CODE
  xprogtyp     = 0  &&    TYPE OF PROGRAM EXECUTION (E-MAIL OR BATCH)
  
  *************************** SCREEN LAYOUT ***************************
  *                         BROWSE TIMER RECORDS
  *
  *     RECORD NUMBER: 999          TIMER CODE: 99          PROGRAM CODE: 9
  *
  *              TIMER SETTIMG TIME: 99:99  DATE: 99/99/99
  *              LAST ACTIVITY TIME: 99:99  DATE: 99/99/99
  *
  *     DAY OF WEEK: 9            DAY OF MONTH: 99        MONTH OF YEAR: 999
  *  --------------------------------------------------------------------------
  *B|
  *A|     BATCH FILE TEXT AREA
  *T|
  *C|
  *H|
  * |
  * |--------------------------------------------------------------------------
  * |
  *E|       E-MAIL TEXT AREA
  *M|
  *A|
  *I|
  *L|
  * |
  *  --------------------------------------------------------------------------
  *****************************************************************************
  
  SELECT 1
  
  GO TOP
  
  IF EOF()=.T.
   APPEND BLANK
  ENDIF
  
  xdelete=.F.		&& LOCAL FLAG
  trecno=0
  
  SET TALK OFF
  SET EXACT ON
  SET CONFIRM ON
  CLEAR
  
  *=============================================================================
  * PLACE LITERALS ON THE SCREEN ( @ n,n SAY "...")
  
  SET COLOR TO GR+/R
  @ 4,1 SAY "F1=HELP"
  @ 0,26 SAY "BROWSE TIMER RECORDS"
  SET COLOR TO GR+/BG,GR+/N,BG
  @ 2,6 SAY "RECORD NUMBER"
  @ 2,34 SAY "TIMER CODE:"
  @ 2,58 SAY "PROGRAM CODE:"
  @ 4,15 SAY "TIMER SETTING TIME:"
  @ 4,42 SAY "DATE:"
  @ 5,15 SAY "LAST ACTIVITY TIME:"
  @ 5,42 SAY "DATE:"
  @ 7,6 SAY "DAY OF WEEK:"
  @ 7,32 SAY "DAY OF MONTH:"
  @ 7,58 SAY "MONTH OF YEAR:"
  @ 9,1  SAY "B"
  @ 10,1 SAY "A"
  @ 11,1 SAY "T"
  @ 12,1 SAY "C"
  @ 13,1 SAY "H"
  @ 17,1 SAY "E"
  @ 18,1 SAY "M"
  @ 19,1 SAY "A"
  @ 20,1 SAY "I"
  @ 21,1 SAY "L"
  
  *=============================================================================
  
  DO WHILE .T.		&& ENTER SEVERAL NUMBERS
   
   IF LASTKEY()=27
    EXIT
   ENDIF
   
   SET COLOR TO GR+/R
   @ 24,1 SAY "Return=EDIT   Pg.Up=PRIOR   Pg.Down=NEXT    Ins=ADD    DeL=DELETE     Esc=QUIT"
   SET COLOR TO GR+/BG,G+/N,BG
   DO WHILE EOF()=.F.		&& SCREEN ENTRY LOOP
    DO WHILE .T. 			&& INKEY LOOP
     
     *=============================================================================
     *XFER DATA FROM FIELD VARIABLES TO SCREEN VARIABLES (XVAR=VAR)
     
     
     xalrmtime    =  a->alrmtime
     xalrmdate    =  a->alrmdate
     xlasttime    =  a->lasttime
     xlastdate    =  a->lastdate
     xmonnumy     =  a->monnumy
     xdaynumm     =  a->daynumm
     xdaynumw     =  a->daynumw
     xnotifyc     =  a->notifyc
     xprogtyp     =  a->progtyp
     
     *=============================================================================
     
     SET COLOR TO GR+/N
     
     *=============================================================================
     *DISPLAY SCREEN VARIABLES (@ N,N SAY XVAR PICT "..." )
     
     @ 2,21 SAY RECNO()    PICT "999"
     @ 0,70 SAY TIME() PICT "99:99"
     
     @ 2,46 SAY xnotifyc   PICT "99"
     @ 2,72 SAY xprogtyp   PICT "9"
     @ 4,35 SAY xalrmtime  PICT "99:99"
     @ 4,48 SAY xalrmdate  PICT "@D"
     @ 5,35 SAY xlasttime  PICT "99:99"
     @ 5,48 SAY xlastdate  PICT "@D"
     @ 7,19 SAY xdaynumw   PICT "9"
     @ 7,45 SAY xdaynumm   PICT "99"
     @ 7,73 SAY xmonnumy   PICT "99"
     
     SET COLOR TO N/W,G+/N,BG
     KEYBOARD CHR(27)
     Memoedit(a->batcht,8,3,15,78,.T.,"",250)
     
     SET COLOR TO W+/B,G+/N,BG
     KEYBOARD CHR(27)
     Memoedit(a->emailt,16,3,23,78,.T.,"",250)
     SET COLOR TO GR+/BG,G+/N,BG
     *=============================================================================
     
     IF DELETED()=.T.
      SET COLOR TO GR+/R
      @ 0,3 SAY "RECORD DELETED"
     ELSE
      SET COLOR TO GR+/BG
      @ 0,1 SAY SPACE(20)
     ENDIF
     SET COLOR TO GR+/BG
     qscr=.F.
     keypress=INKEY(60)  && GET KEY INPUT & UPDATE SCREEN ONCE PER SECOND
     
     DO CASE
     CASE keypress= 28           && CALL HELP SCREEN
      call_prg="TIMER.PRG"
      line_num=0
      input_var=""
      DO HELP WITH call_prg, line_num
     CASE keypress= 27           && ESCAPE KEY EXITS THE PROGRAM
      qscr=.T.
      EXIT
     CASE keypress= 18           && PAGE UP, PRIOR SCREEN
      SKIP -1
      IF BOF()=.T.
       GO BOTTOM
      ENDIF
     CASE keypress= 3            && PAGE DOWN, NEXT SCREEN
      SKIP 1
      IF EOF()=.T.
       GO TOP
      ENDIF
     CASE keypress=13		&& RETURN = EDIT SCREEN
      EXIT
     CASE keypress=7		&& DELETE KEY DELETES THE RECORD
      IF RECNO()<>1
       IF DELETED()=.T.		&& IF ALREADY DELETED, THEN UN-DELETE
        IF rec_lock(5)
         RECALL
         UNLOCK
        ELSE
         SET COLOR TO GR+*/R
         @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
         SET COLOR TO GR+/BG,G+/N,BG
         INKEY(0)
         @ 0,0
        ENDIF
        SET COLOR TO GR+/BG,G+/N,BG
        @ 0,1 SAY SPACE(20)
       ELSE                        && IF NOT ALREADY DELETED, THEN DELETE
        IF rec_lock(5)
         DELETE
         UNLOCK
        ELSE
         SET COLOR TO GR+*/R
         @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
         SET COLOR TO GR+/BG,G+/N,BG
         INKEY(0)
         @ 0,0
        ENDIF
        xdelete=.T.
        SET COLOR TO GR+/R
        @ 0,3 SAY "RECORD DELETED"
        SET COLOR TO GR+/BG,G+/N,BG
       ENDIF
      ENDIF
     CASE keypress=22		&& INSERT KEY TO ADD NEW RECORDS
      SET COLOR TO GR+/BG,G+/N,BG
      @ 0,1 SAY SPACE(20)
      yn=" "
      SET COLOR TO GR+/R,G+/ ,BG
      @ 0,1 SAY "EDITING NEW RECORD"
      @ 24,1 SAY "   TYPE CONTROL 'W' TO STORE THE TEXT WINDOW, OR ESC TO LEAVE UNCHANGED.      "
      
      SET COLOR TO GR+/BG,G+/N,BG
      
      * COPY MEMO FIELDS FROM CURRENT RECORD, OR TEXT FILES, TO THE NEW RECORD
      
      IF FILE(ibatpath)=.T.
       xbatcht = Memoread(ibatpath)
       ERASE &ibatpath
      ELSE
       xbatcht = a->batcht
      ENDIF
      
      IF FILE(iempath)=.T.
       xemailt = Memoread(iempath)
       ERASE &iempath
      ELSE
       xemailt = a->emailt
      ENDIF
      
      SET COLOR TO N/W,G+/N,BG
      KEYBOARD CHR(27)
      Memoedit(xbatcht,8,3,15,78,.T.,"",250)
      
      SET COLOR TO W+/B,G+/N,BG
      KEYBOARD CHR(27)
      Memoedit(xemailt,16,3,23,78,.T.,"",250)
      SET COLOR TO GR+/BG,G+/N,BG
      
      IF add_rec(5)=.F.		&& APPEND RECORD
       SET COLOR TO GR+*/R
       @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
       SET COLOR TO GR+/BG,G+/N,BG
       INKEY(0)
       @ 0,0
      ENDIF
      
      @ 2,21 SAY RECNO()    PICT "999"
      
      *=============================================================================
      *XFER DATA FROM FIELD VARIABLES TO SCREEN VARIABLES (XVAR=VAR)
      
      xalrmtime    =  a->alrmtime
      xalrmdate    =  a->alrmdate
      xlasttime    =  a->lasttime
      xlastdate    =  a->lastdate
      xmonnumy     =  a->monnumy
      xdaynumm     =  a->daynumm
      xdaynumw     =  a->daynumw
      xnotifyc     =  a->notifyc
      xprogtyp     =  a->progtyp
      
      *=============================================================================
      * GET VARIABLES FROM SCREEN ( @ n,n GET XVARS PICT "...")
      
      rechold=RECNO()
      
      
      @ 2,46 GET xnotifyc   PICT "99"
      @ 2,72 GET xprogtyp   PICT "9"
      @ 4,35 GET xalrmtime  PICT "99:99"
      @ 4,48 GET xalrmdate  PICT "@D"
      @ 5,35 GET xlasttime  PICT "99:99"
      @ 5,48 GET xlastdate  PICT "@D"
      @ 7,19 GET xdaynumw   PICT "9"
      @ 7,45 GET xdaynumm   PICT "99"
      @ 7,73 GET xmonnumy   PICT "99"
      READ
      
      IF RECNO()=1
       xalrmtime    = SPACE(5)   &&      ALARM TIME
       xalrmdate    = CTOD("  /  /  ")  &&      ALARM DATE
       xlasttime    = SPACE(5)    &&      TIME OF LAST ACTIVITY
       xlastdate    = CTOD("  /  /  ")  &&      DATE OF LAST ACTIVITY
       xmonnumy     = 0  &&   MONTH OF YEAR HOLDING VARIABLE
       xdaynumm     = 0  &&   DAY OF MONTH HOLDING VARIABLE
       xdaynumw     = 0  &&   DAY OF WEEK HOLDING VARIABLE
       xnotifyc     = 0  &&    METHOD OF NOTIFICATION CODE
       xprogtyp     = 0  &&    TYPE OF PROGRAM EXECUTION (E-MAIL OR BATCH)
      ENDIF
      
      SET COLOR TO N/W,G+/N,BG
      xbatcht=Memoedit(xbatcht,8,3,15,78,.T.,"",250)
      SET COLOR TO W+/B,G+/N,BG
      xemailt=Memoedit(xemailt,16,3,23,78,.T.,"",250)
      SET COLOR TO GR+/BG,G+/N,BG
      
      
      SET COLOR TO G+/N
      *=============================================================================
      * XFER DATA FROM SCREEN VARIABLES TO FIELD VARIABLES (REPLACE VAR WITH XVAR)
      
      
      IF rec_lock(5)
       REPLACE a->alrmtime    WITH xalrmtime
       REPLACE a->alrmdate    WITH xalrmdate
       REPLACE a->lasttime    WITH xlasttime
       REPLACE a->lastdate    WITH xlastdate
       REPLACE a->monnumy     WITH xmonnumy
       REPLACE a->daynumm     WITH xdaynumm
       REPLACE a->daynumw     WITH xdaynumw
       REPLACE a->notifyc     WITH xnotifyc
       REPLACE a->progtyp     WITH xprogtyp
       REPLACE a->batcht WITH xbatcht
       REPLACE a->emailt WITH xemailt
       COMMIT
       UNLOCK
      ELSE
       SET COLOR TO GR+*/R
       @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
       SET COLOR TO GR+/BG,G+/N,BG
       INKEY(0)
       @ 0,0
      ENDIF
      
      *=============================================================================
      
      
     ENDCASE
     
     SET COLOR TO GR+/R,G+/ ,BG
     @ 24,1 SAY "Return=EDIT   Pg.Up=PRIOR   Pg.Down=NEXT    Ins=ADD    DeL=DELETE     Esc=QUIT"
     SET COLOR TO GR+/BG,G+/N,BG
     
     
    ENDDO 				&& INKEY LOOP
    IF qscr=.T.
     EXIT
    ENDIF
    IF DELETED()=.T.
     SET COLOR TO GR+/R
     @ 0,3 SAY "RECORD DELETED"
    ELSE
     SET COLOR TO GR+/BG,G+/N,BG
     @ 0,1 SAY SPACE(20)
     
    ENDIF
    
    SET COLOR TO GR+/R,G+/ ,BG
    @ 24,1 SAY "   TYPE CONTROL 'W' TO STORE THE TEXT WINDOW, OR ESC TO LEAVE UNCHANGED.      "
    
    SET COLOR TO G+/N
    *=============================================================================
    * GET VARIABLES FROM SCREEN ( @ n,n GET XVARS PICT "...")
    
    @ 2,46 GET xnotifyc   PICT "99"
    @ 2,72 GET xprogtyp   PICT "9"
    @ 4,35 GET xalrmtime  PICT "99:99"
    @ 4,48 GET xalrmdate  PICT "@D"
    @ 5,35 GET xlasttime  PICT "99:99"
    @ 5,48 GET xlastdate  PICT "@D"
    @ 7,19 GET xdaynumw   PICT "9"
    @ 7,45 GET xdaynumm   PICT "99"
    @ 7,73 GET xmonnumy   PICT "99"
    READ
    
    IF RECNO()=1
     xalrmtime    = SPACE(5)   &&      ALARM TIME
     xalrmdate    = CTOD("  /  /  ")  &&      ALARM DATE
     xlasttime    = SPACE(5)    &&      TIME OF LAST ACTIVITY
     xlastdate    = CTOD("  /  /  ")  &&      DATE OF LAST ACTIVITY
     xmonnumy     = 0  &&   MONTH OF YEAR HOLDING VARIABLE
     xdaynumm     = 0  &&   DAY OF MONTH HOLDING VARIABLE
     xdaynumw     = 0  &&   DAY OF WEEK HOLDING VARIABLE
     xnotifyc     = 0  &&    METHOD OF NOTIFICATION CODE
     xprogtyp     = 0  &&    TYPE OF PROGRAM EXECUTION (E-MAIL OR BATCH)
    ENDIF
    
    SET COLOR TO N/W,G+/N,BG
    xbatcht=Memoedit(a->batcht,8,3,15,78,.T.,"",250)
    SET COLOR TO W+/B,G+/N,BG
    xemailt=Memoedit(a->emailt,16,3,23,78,.T.,"",250)
    SET COLOR TO GR+/BG,G+/N,BG
    
    *=============================================================================
    * XFER DATA FROM SCREEN VARIABLES TO FIELD VARIABLES (REPLACE VAR WITH XVAR)
    
    IF rec_lock(5)
     REPLACE a->alrmtime    WITH xalrmtime
     REPLACE a->alrmdate    WITH xalrmdate
     REPLACE a->lasttime    WITH xlasttime
     REPLACE a->lastdate    WITH xlastdate
     REPLACE a->monnumy     WITH xmonnumy
     REPLACE a->daynumm     WITH xdaynumm
     REPLACE a->daynumw     WITH xdaynumw
     REPLACE a->notifyc     WITH xnotifyc
     REPLACE a->progtyp     WITH xprogtyp
     REPLACE a->batcht WITH xbatcht
     REPLACE a->emailt WITH xemailt
     COMMIT
     UNLOCK
    ELSE
     SET COLOR TO GR+*/R
     @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
     SET COLOR TO GR+/BG,G+/N,BG
     INKEY(0)
     @ 0,0
    ENDIF
    
    SET COLOR TO GR+/R,G+/ ,BG
    @ 24,1 SAY "Return=EDIT   Pg.Up=PRIOR   Pg.Down=NEXT    Ins=ADD    DeL=DELETE     Esc=QUIT"
    SET COLOR TO GR+/BG,G+/N,BG
    
    
    *=============================================================================
    
   ENDDO		&& SCREEN ENTRY LOOP
   
  ENDDO			 && ENTER SEVERAL RECORDS
  
  
  IF xdelete=.T.
   yn="N"
   CLEAR
   @ 10,5 SAY "DO YOU WANT TO ERASE ALL RECORDS MARKED FOR DELETION (Y/N)? " ;
    GET yn PICT "!" VALID yn="Y" .OR. yn="N"
   READ
   CLOSE
   IF yn="Y"
    IF net_use(filepath, .T., 5)
     PACK
     CLOSE
    ELSE
     SET COLOR TO GR+*/R
     @ 0,0 SAY "FILE ALREADY IN USE!     HIT ANY KEY TO CONTINUE!"
     SET COLOR TO GR+/BG,G+/N,BG
     INKEY(0)
     @ 0,0
    ENDIF
   ENDIF
   CLOSE
   IF net_use(filepath, .F., 5)
    *
   ELSE
    SET COLOR TO GR+*/R
    @ 0,0 SAY "FILE ALREADY IN USE!     HIT ANY KEY TO CONTINUE!"
    SET COLOR TO GR+/BG,G+/N,BG
    INKEY(0)
    @ 0,0
    RETURN
   ENDIF
  ENDIF
  
  CLEAR
  SET DELETED ON
  
  
 ENDCASE  && TEST FOR MENU SELECTION
 
ENDDO     && MENU LOOP

SELECT 1
CLOSE



*!*********************************************************************
*!
*!      Procedure: NOTIFY
*!
*!      Called by: TIMER.PRG                     
*!
*!          Calls: REC_LOCK()     (function  in TIMER.PRG)
*!
*!           Uses: &BATPATH.DBF   
*!               : &EMPATH.DBF    
*!
*!*********************************************************************
PROCEDURE notify


DO CASE	&& TEST FOR TYPE OF NOTIFICATION
 
CASE xprogtyp = 1	&& SEND E-MAIL LETTER, BODY FROM MEMO FIELD
 ERASE &batpath
 Memowrit(batpath,a->batcht)  && COPY BATCH MEMO TO BATCH FILE
 ERASE &empath
 Memowrit(empath,a->emailt)   && COPY EMAIL MEMO TO TMP.TXT
 
 IF rec_lock(5)
  REPLACE a->lasttime    WITH SUBSTR(TIME(),1,5)
  REPLACE a->lastdate    WITH DATE()
  COMMIT
  UNLOCK
 ELSE
  SET COLOR TO GR+*/R
  @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
  SET COLOR TO GR+/BG,G+/N,BG
  INKEY(0)
  @ 0,0
 ENDIF
 
 IF FILE(batpath)=.T. .AND. FILE(empath)=.T.
  SELECT 1
  CLOSE
  QUIT 	&& TERMINATE PROGRAM AND EXECUTE TIMECONT.BAT
 ENDIF
 
 
CASE xprogtyp = 2	 && SEND E-MAIL LETTER, BODY FROM TEXT FILE
 ERASE &batpath
 Memowrit(batpath,a->batcht)  && COPY BATCH MEMO TO BATCH FILE
 * BATCH FILE WILL IMPORT A TEXT FILE INTO THE BODY OF THE LETTER
 
 IF rec_lock(5)
  REPLACE a->lasttime    WITH SUBSTR(TIME(),1,5)
  REPLACE a->lastdate    WITH DATE()
  COMMIT
  UNLOCK
 ELSE
  SET COLOR TO GR+*/R
  @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
  SET COLOR TO GR+/BG,G+/N,BG
  INKEY(0)
  @ 0,0
 ENDIF
 
 IF FILE(batpath)=.T. .AND. FILE(empath)=.T.
  SELECT 1
  CLOSE
  QUIT 	&& TERMINATE PROGRAM AND EXECUTE TIMECONT.BAT
 ENDIF
 
CASE xprogtyp = 3 && RUN BATCH FILE ONLY
 ERASE &batpath
 Memowrit(batpath,a->batcht)  && COPY BATCH MEMO TO BATCH FILE
 
 IF rec_lock(5)
  REPLACE a->lasttime    WITH SUBSTR(TIME(),1,5)
  REPLACE a->lastdate    WITH DATE()
  COMMIT
  UNLOCK
 ELSE
  SET COLOR TO GR+*/R
  @ 0,0 SAY "RECORD ALREADY IN USE!      HIT ANY KEY TO CONTINUE!"
  SET COLOR TO GR+/BG,G+/N,BG
  INKEY(0)
  @ 0,0
 ENDIF
 
 IF FILE(batpath)=.T.
  SELECT 1
  CLOSE
  QUIT 	&& TERMINATE PROGRAM AND EXECUTE TIMECONT.BAT
 ENDIF
 
ENDCASE	&& END TEST FOR TYPE OF NOTIFICATION

RETURN



*!*********************************************************************
*!
*!      Procedure: HELP
*!
*!      Called by: TIMER.PRG                     
*!
*!          Calls: MEMOFUNC2()    (function  in TIMER.PRG, called from MemoEdit())
*!               : REC_LOCK()     (function  in TIMER.PRG)
*!
*!*********************************************************************
PROCEDURE HELP

PARAMETERS mproc, mline_no, mvar
PRIVATE cur_file, target, memo_fld, ROW, COL, edit_it

IF mproc = "HELP"
 RETURN
ENDIF

savehelp=RECNO()
GO TOP		&& ALWAYS USE RECORD # 1

ins_on=.F.
scrl_on=.F.
deja_vu=.F.
init_count=1
word_wrap=.T.
UPDATE=.T.

CLEAR GETS

IF Pcount() < 3
 mproc = ""
 mline_no = 0
 mvar = ""
ENDIF

htr = 04
htc = 05
hbr = 21
hbc = 74

ROW = ROW()
COL = COL()

SET COLOR TO GR+/N,G+/N,N

SAVE SCREEN

* THE FILE SHOULD ALREADY BE OPEN

@ htr, htc CLEAR TO hbr, hbc	&& CLEAR A WINDOW
@ htr-1,htc-1 TO hbr+1,hbc+1 DOUBLE

@ htr-1,htc-1 TO hbr+1,hbc+1 DOUBLE
@ 24,0

@ hbr+1,24 SAY " HIT ESC TO QUIT HELP COMMENTS "
SET COLOR TO W+/N,G+/N,N


xhelp = Memoedit(a->help, htr, htc, hbr, hbc, .T.,"MEMOFUNC2")

IF rec_lock(5)
 REPLACE a->help WITH xhelp
 COMMIT
 UNLOCK
ELSE
 ? "Record not available"
 INKEY(5)
ENDIF
KEYBOARD " "
INKEY(.1)
RESTORE SCREEN

SET COLOR TO GR+/BG,G+/N,N
GOTO savehelp
RETURN



*!*********************************************************************
*!
*!       Function: MEMOFUNC2()
*!
*!      Called by: HELP           (procedure in TIMER.PRG)
*!
*!*********************************************************************
FUNCTION memofunc2
PARAMETERS mode,Line,COL

DO CASE
CASE mode=0
 
 IF LASTKEY()=27
  RETURN 23
 ENDIF
 
 RETURN 0
 
CASE mode=1
 
 DO CASE
 CASE LASTKEY()=27
  RETURN 23
  
 ENDCASE
 RETURN 0
 
CASE mode=2
 DO CASE
 CASE LASTKEY()=27
  RETURN 23
 ENDCASE
 RETURN 0
 
CASE mode=3
 ret_val=0
 
 IF init_count = 1
  ins_mode = Readinsert()
  IF (ins_on .AND. .NOT. ins_mode) .OR.;
    (.NOT. ins_on .AND. ins_mode)
   ret_val = 22
  ELSE
   init_count = 2
  ENDIF
 ENDIF
 
 IF init_count = 2
  IF ((.NOT. scrl_on .AND. .NOT. UPDATE) .OR.;
    (scrl_on .AND. UPDATE)) .AND. .NOT. deja_vu
   deja_vu = .T.
   ret_val = 35
  ELSE
   init_count = 3
   deja_vu = .F.
  ENDIF
 ENDIF
 
 IF init_count = 3
  IF .NOT. word_wrap .AND. .NOT. deja_vu
   deja_vu = .T.
   ret_val = 34
  ELSE
   init_count = 4
   deja_vu = .F.
  ENDIF
 ENDIF
 
 IF init_count = 4
  ret_val = 0
 ENDIF
 
 RETURN ret_val
 
ENDCASE
RETURN 0

******************************************************************************
*
*  Assumes SET EXCLUSIVE OFF
*
*   UDF's for:
*   Command   			Purpose
*   -----------------------	------------------------
*   1. USE <file> [EXCLUSIVE]	Share/Deny others USE
*   2. FLOCK()			Lock current file
*   3. RLOCK()/LOCK()		Lock current record
*   4. APPEND BLANK		Lock new blank record


****
*  NET_USE function
*
*  Trys to open a file for exclusive or shared use.
*  SET INDEXes in calling procedure if successful.
*  Pass the following parameters
*    1. Character - name of the .DBF file to open
*    2. Logical - mode of open (exclusive/.NOT. exclusive)
*    3. Numeric - seconds to wait (0 = wait forever)
*
*  Example:
*    IF NET_USE("Accounts", .T., 5)
*       SET INDEX TO Name
*    ELSE
*       ? "Account file not available"
*    ENDIF

*!*********************************************************************
*!
*!       Function: NET_USE()
*!
*!      Called by: TIMER.PRG                     
*!
*!           Uses: &FILE          
*!
*!*********************************************************************
FUNCTION net_use
PARAMETERS FILE, ex_use, WAIT
PRIVATE forever

forever = (WAIT = 0)
DO WHILE (forever .OR. WAIT > 0)
 
 IF ex_use                    && exclusive
  USE &file EXCLUSIVE
 ELSE
  USE &file   		&& shared
 ENDIF
 
 IF .NOT. Neterr()           && USE succeeds
  RETURN (.T.)
 ENDIF
 
 INKEY(1) 	          	&& wait 1 second
 WAIT = WAIT - 1
ENDDO
RETURN (.F.)			&& USE fails
* End - NET_USE


****
*  FIL_LOCK function
*
*  Trys to lock the current shared file
*  Pass the following parameter
*    1. Numeric - seconds to wait (0 = wait forever)
*
* Example:
*    IF FIL_LOCK(5)
*      REPLACE ALL Price WITH Price * 1.1
*    ELSE
*      ? "File not available"
*    ENDIF

*!*********************************************************************
*!
*!       Function: FIL_LOCK()
*!
*!*********************************************************************
FUNCTION fil_lock
PARAMETERS WAIT
PRIVATE forever

IF FLOCK()
 RETURN (.T.)		&& locked
ENDIF

forever = (WAIT = 0)
DO WHILE (forever .OR. WAIT > 0)
 
 INKEY(.5)			&& wait 1/2 second
 WAIT = WAIT - .5
 
 IF FLOCK()
  RETURN (.T.)		&& locked
 ENDIF
 
ENDDO
RETURN (.F.)			&& not locked
* End - FIL_LOCK


****
*  REC_LOCK function
*
*  Trys to lock the current record
*  Pass the following parameter
*    1. Numeric - seconds to wait (0 = wait forever)
*
* Example:
*   IF REC_LOCK(5)
*      REPCLACE Price WITH newprice
*   ELSE
*      ? "Record not available"
*   ENDIF

*!*********************************************************************
*!
*!       Function: REC_LOCK()
*!
*!      Called by: TIMER.PRG                     
*!               : HELP           (procedure in TIMER.PRG)
*!               : NOTIFY         (procedure in TIMER.PRG)
*!
*!*********************************************************************
FUNCTION rec_lock
PARAMETERS WAIT
PRIVATE forever

IF RLOCK()
 RETURN (.T.)		&& locked
ENDIF

forever = (WAIT = 0)
DO WHILE (forever .OR. WAIT > 0)
 
 IF RLOCK()
  RETURN (.T.)		&& locked
 ENDIF
 
 INKEY(.5)			&& wait 1/2 second
 WAIT = WAIT - .5
 
ENDDO
RETURN (.F.)			&& not locked
* End - REC_LOCK


****
*   ADD_REC function
*
*  Returns true if record appended.  The new record is current
*  and locked.
*  Pass the following parameter
*    1. Numeric - seconds to wait (0 = wait forever)
*

*!*********************************************************************
*!
*!       Function: ADD_REC()
*!
*!      Called by: TIMER.PRG                     
*!
*!*********************************************************************
FUNCTION add_rec
PARAMETERS WAIT
PRIVATE forever

APPEND BLANK
IF .NOT. Neterr()
 RETURN (.T.)
ENDIF

forever = (WAIT = 0)
DO WHILE (forever .OR. WAIT > 0)
 
 APPEND BLANK
 IF .NOT. Neterr()
  RETURN .T.
 ENDIF
 
 INKEY(.5)			&& wait 1/2 second
 WAIT = WAIT - .5
 
ENDDO
RETURN (.F.)			&& not locked
* End ADD_REC

**************************************************************************
*: EOF: TIMER.PRG
